import asyncio
import logging
import platform
from contextlib import contextmanager

from pika.adapters import asyncio_connection
from pika import channel

from .version import __version__


LOGGER = logging.getLogger(__name__)

PRODUCT = 'aio-pika'


class AsyncioConnection(asyncio_connection.AsyncioConnection):

    def __init__(self, parameters=None, on_open_callback=None,
                 on_open_error_callback=None, on_close_callback=None,
                 stop_ioloop_on_close=False, loop: asyncio.AbstractEventLoop=None):

        super().__init__(
            parameters=parameters,
            on_open_callback=on_open_callback,
            on_open_error_callback=on_open_error_callback,
            on_close_callback=on_close_callback,
            stop_ioloop_on_close=stop_ioloop_on_close,
            custom_ioloop=loop,
        )

        self.channel_cleanup_callback = None

    @property
    def _client_properties(self) -> dict:
        """ Return the client properties dictionary. """
        return {
            'product': PRODUCT,
            'platform': 'Python %s' % platform.python_version(),
            'capabilities': {
                'authentication_failure_close': True,
                'basic.nack': True,
                'connection.blocked': True,
                'consumer_cancel_notify': True,
                'publisher_confirms': True
            },
            'information': 'See https://aio-pika.readthedocs.io/',
            'version': __version__
        }

    def _on_channel_cleanup(self, channel):
        try:
            if self.channel_cleanup_callback:
                self.channel_cleanup_callback(channel)
        finally:
            super()._on_channel_cleanup(channel)

    def _create_channel(self, channel_number, on_open_callback):
        LOGGER.debug('Creating channel %s', channel_number)
        return Channel(self, channel_number, on_open_callback)


class Channel(channel.Channel):
    def __init__(self, connection, channel_number, on_open_callback=None):
        super().__init__(connection, channel_number, on_open_callback=on_open_callback)
        self._consume_results = {}
        self._on_getempty_callback = None

    def _on_eventok(self, method_frame):
        callback = self._consume_results.pop(method_frame.method.consumer_tag, None)

        if callback:
            callback(method_frame)

        return super()._on_eventok(method_frame)

    def basic_consume(self, consumer_callback,
                      queue='',
                      no_ack=False,
                      exclusive=False,
                      consumer_tag=None,
                      arguments=None,
                      result_callback=None):

        if not consumer_tag:
            consumer_tag = self._generate_consumer_tag()

        self._consume_results[consumer_tag] = result_callback

        return super().basic_consume(
            consumer_callback=consumer_callback,
            queue=queue,
            no_ack=no_ack,
            exclusive=exclusive,
            consumer_tag=consumer_tag,
            arguments=arguments
        )

    def _on_getempty(self, method_frame):
        if self._on_getempty_callback:
            self._on_getempty_callback(method_frame)
        else:
            LOGGER.debug("Unexcpected getempty frame %r", method_frame)

        super()._on_getempty(method_frame)

    @contextmanager
    def set_get_empty_callback(self, callback):
        self._on_getempty_callback = callback
        try:
            yield
        finally:
            self._on_getempty_callback = None



import os
from setuptools import setup, find_packages
from importlib.machinery import SourceFileLoader


module = SourceFileLoader("version", os.path.join("aio_pika", "version.py")).load_module()


setup(
    name='aio-pika',
    version=module.__version__,
    author=module.__author__,
    author_email=module.team_email,
    license=module.package_license,
    description=module.package_info,
    long_description=open("README.rst").read(),
    platforms="all",
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Internet',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: MacOS',
        'Operating System :: POSIX',
        'Operating System :: Microsoft',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    packages=find_packages(exclude=['tests']),
    install_requires=[
        'shortuuid',
        'pika>=0.11,<0.12',
        'yarl',
    ],
    python_requires=">3.4.*, <4",
    extras_require={
        'develop': [
            'asynctest<0.11',
            'coverage!=4.3',
            'coveralls',
            'pylama',
            'pytest',
            'pytest-asyncio<0.6',
            'pytest-cov',
            'sphinx',
            'sphinx-autobuild',
            'timeout-decorator',
            'tox>=2.4',
        ],
        ':python_version < "3.5"': 'typing >= 3.5.3',
    },
)
